try:
    import PIL.Image
    import PIL.ImageFilter
except ImportError: pass
import sys, string

from cStringIO import StringIO
from M2Crypto import Rand, SSL, httpslib
import urllib
from xml.dom.minidom import Document
from xml.dom.minidom import parse, parseString
import xml.dom.minidom
import csv

def ImageResize(origimg_binstr, width, height, engine='ImageMagick', quality=75):
    """Resize and resample image."""
    newimg = StringIO()
    if engine == 'PIL':  # Use PIL
        origimg=StringIO(origimg_binstr)
        img = PIL.Image.open(origimg)
        fmt = img.format
        img = img.convert('RGB')
        img = img.filter(PIL.ImageFilter.SMOOTH)
        img = img.resize((width, height))
        img = img.filter(PIL.ImageFilter.SMOOTH)
        img.save(newimg, fmt, quality=quality)
    elif engine == 'ImageMagick':  # Use ImageMagick
        if sys.platform == 'win32':
            from win32pipe import popen2
            imgin, imgout = popen2('convert -quality %s -geometry %sx%s - -'
                                   % (quality, width, height), 'b')
        else:
            from popen2 import popen2
            imgout, imgin = popen2('convert -quality %s -geometry %sx%s - -'
                                   % (quality, width, height))
        imgin.write(origimg_binstr)
        imgin.close()
        newimg.write(imgout.read())
        imgout.close()

    newimg.seek(0)
    returnimg = newimg.read()
    return returnimg

def ImageConvert(origimg_binstr,dst_fmt,quality=75,engine='PIL'):
    """ Convert image from one format to another """
    newimg = StringIO()
    if engine == 'PIL':  # Use PIL
        origimg=StringIO(origimg_binstr)
        img = PIL.Image.open(origimg)
        if img.format==dst_fmt:
            return origimg_binstr
        img.draft('RBG',img.size)
        img = img.convert('RGB')
        img.save(newimg,dst_fmt,quality=quality)
    elif engine == 'ImageMagick':
        newfmt='GIF'
        for fmt in ('GIF','JPEG','PNG'):
            if fmt==dst_fmt:
                newfmt=dst_fmt
                break
                
        if sys.platform == 'win32':
            from win32pipe import popen2
            imgin, imgout = popen2('convert - %s:-' % newfmt)
        else:
            from popen2 import popen2
            imgout, imgin = popen2('convert - %s:-' % newfmt)
            
        imgin.write(origimg_binstr)
        imgin.close()
        newimg.write(imgout.read())
        imgout.close()
        

    
    newimg.seek(0)
    returnimg = newimg.read()
    return returnimg
        

def ImageInfo(origimg_binstr,engine='PIL'):
    """ Returns image size,format calculated """
    if engine == 'PIL':  # Use PIL
        origimg=StringIO(origimg_binstr)
        img = PIL.Image.open(origimg)
        format = img.format
        size = img.size
        return (format,size)
    elif engine=='ImageMagick':
        newimg = StringIO()
        if sys.platform == 'win32':
            from win32pipe import popen2
            imgin, imgout = popen2('identify -format "%m,%w,%h," -')
        else:
            from popen2 import popen2
            imgout, imgin = popen2('identify -format "%m,%w,%h," -')

        imgin.write(origimg_binstr)
        imgin.close()
        my_arr = string.split(imgout.read(),',')
        imgout.close()
        return (my_arr[0],(int(my_arr[1]),int(my_arr[2])))
    else:
        raise ValueError,'engine not defined: '+engine
    
#input=open("in.gif","r")
#input_str=input.read()
#input.close()
#print ImageInfo(input_str,engine='ImageMagick')
#out_str=ImageConvert(input_str,'JPEG',engine='PIL')
#out_str=ImageResize(input_str,300,240,engine='PIL')
#out=open('out.gif','w')
#out.write(out_str)
#out.close()


# HTTPS client for UPS
def HTTPSRequest (host,port,action,method,params,urlencode_params=0):
    ctx = SSL.Context()
    h = httpslib.HTTPSConnection(host, port, ssl_context=ctx)
    h.set_debuglevel(1)
    headers = {"Content-type": "application/x-www-form-urlencoded"}
    #headers = {"Content-type": "application/xml"}
    #headers = {"Content-type": "text/xml"}
    if urlencode_params:
        h.request(method,action,urllib.urlencode(params), headers)
    else:
        h.request(method,action,params, headers)
    response = h.getresponse()
    
    # Comments by: Alex, 2003-06-23
    #
    # I do not understand why UPS server does not contains Content-length
    # field now. In time of writing this extension this header was present.
    # and then, suddenly it disappeared, as result our customer stores
    # stopped working on big XML responses... 
    # 
    # That is how it worked before:
    # return (1, response.read(int(response.getheader('Content-length'))))
    # 
    # Now re-implementation with 50Kb buffer:
    data = ''
    while(1):
        my_buffer = response.read(50000) # 50 KB buffer
        if len(my_buffer)==0:
            break
        data = data + my_buffer
    
    return (1, data)


def urlOpen(url,params=None):
    if params is not None:
        params = urllib.urlencode(params)
        f = urllib.urlopen(url,params)
    else:
        f = urllib.urlopen(url)
    data=f.read()
    f.close()
    return data
    

def handleUPS_Error(error):
    error_severity = getText(error.getElementsByTagName('ErrorSeverity')[0].childNodes)
    error_code = getText(error.getElementsByTagName('ErrorCode')[0].childNodes)
    error_description = getText(error.getElementsByTagName('ErrorDescription')[0].childNodes)
    return (0,{'severity':error_severity,'code':error_code,'descr':error_description})

def handleResponse(resp_s):
    print "RESPONSE: ",resp_s
    try:
        response = parseString(resp_s)
    except:
        return (0,{'severity': hard, 'code': '3000001', 'descr':'XML parser error'})


    if response.documentElement.tagName!='RatingServiceSelectionResponse':
        return {'error':1,'severity':'hard',text:'invalid response from ULS server'}
    Response = response.documentElement.getElementsByTagName('Response')[0]
    response_status_code = getText(Response.getElementsByTagName('ResponseStatusCode')[0].childNodes)
    response_status_description = getText(Response.getElementsByTagName('ResponseStatusDescription')[0].childNodes)
    if response_status_code!='1':
        return handleUPS_Error(Response.getElementsByTagName('Error')[0])
    return (1,Response)

#def UPS_shop_request(host,port,action,method,xml_request):
    
    

def format_number_via_locale(format, number, fmtlocale='en_US'):
    import locale
    locale.setlocale(locale.LC_ALL,fmtlocale)
    return locale.format(format,number,',')
    #return locale.localeconv()


def getText(nodelist):
    rc = ""
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc = rc + node.data
        return rc


def handleUPSRate_And_Shop(xml_in):
    response = parseString(xml_in)
    if response.documentElement.tagName!='RatingServiceSelectionResponse':
        return {'error':1,'severity':'hard',text:'invalid response from ULS server'}
    Response = response.documentElement.getElementsByTagName('Response')[0]
    response_status_code = getText(Response.getElementsByTagName('ResponseStatusCode')[0].childNodes)
    response_status_description = getText(Response.getElementsByTagName('ResponseStatusDescription')[0].childNodes)
    if response_status_code!='1':
        return handleUPS_Error(Response.getElementsByTagName('Error')[0])

    # prepare the list of shipment options available
    ups_variants = []
    for RatedShipment in response.getElementsByTagName('RatedShipment'):
        Service = RatedShipment.getElementsByTagName('Service')[0]
        service_code = getText(Service.getElementsByTagName('Code')[0].childNodes)
        print 'service code: ',service_code
        TotalCharges = RatedShipment.getElementsByTagName('TotalCharges')[0]
        total_charges_currency_code = getText(TotalCharges.getElementsByTagName('CurrencyCode')[0].childNodes)
        print 'total charges currency: ', total_charges_currency_code
        total_charges_monetary_value = getText(TotalCharges.getElementsByTagName('MonetaryValue')[0].childNodes)
        print 'total charges monetary_value: ', total_charges_monetary_value
        ups_variants.append({'service_code': service_code,
                             'total_charges_currency_code': total_charges_currency_code,
                             'total_charges_monetary_value': total_charges_monetary_value})
        

    return (1, ups_variants)

def CSVparser_init(ms_double_quote=1, field_sep=',' , auto_clear=1):
    p = csv.parser(ms_double_quote, field_sep, auto_clear)
    return p
    
def CSV_parse(parser, line):
    return parser.parse(line)

def urlopen_get(url):
    f = urllib.urlopen(url)
    return f.read()
       
import pickle
def myPickle(object):
    src = StringIO()
    p = pickle.Pickler(src)
    p.dump(object)
    return src.getvalue()

def myUnPickle(data):
    return pickle.Unpickler(StringIO(data) ).load()
