#!/home/zope/bin/python

from xml.dom.minidom import Document
from xml.dom.minidom import parse, parseString
import xml.dom.minidom

error = '''<?xml version="1.0"?>
<RateResponse><Package ID="0"><Error><Number>-2147219040</Number><Source>SOLServerTest;solservertest.CallRateDll</Source><Description>This Information has not been included in this Test Server.</Description><HelpFile></HelpFile><HelpContext></HelpContext></Error></Package></RateResponse>'''

error_multiple_products = '''<?xml version="1.0"?>
<RateResponse>
<Package ID="0">
<Error>
<Number>-2147219040</Number>
<Source>SOLServerTest;solservertest.CallRateDll</Source>
<Description>This Information has not been included in this Test Server.</Description>
<HelpFile></HelpFile><HelpContext></HelpContext>
</Error>
</Package>
<Package ID="22">
<Service>EXPRESS</Service>
<ZipOrigination>20770</ZipOrigination>
<ZipDestination>20852</ZipDestination>
<Pounds>10</Pounds>
<Ounces>0</Ounces>
<Container>None</Container>
<Size>REGULAR</Size>
<Zone>1</Zone>
<Postage>33.65</Postage>
</Package>
</RateResponse>'''

def getText(nodelist):
    rc = ""
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc = rc + node.data
        return rc

def handleUSPSResponse(resp_s):
    #print "RESPONSE: ",resp_s
    try:
        response = parseString(resp_s)
    except:
        return (0,[{'severity': hard, 'code': '3000001', 'descr':'XML parser error'}])

    if response.documentElement.tagName!='RateResponse':
        return (0,[{'code':1,'severity':'hard','descr':'invalid response from USPS server'}])
        
    errors_list = []
    total = 0
    for Package in response.documentElement.getElementsByTagName('Package'):
        product_id = Package.getAttributeNode('ID').value
        try:
            error = Package.getElementsByTagName('Error')[0]
            error_code = getText(error.getElementsByTagName('Number')[0].childNodes)
            error_description = getText(error.getElementsByTagName('Description')[0].childNodes)
            #print product_id, error_code, error_description
            errors_list.append({'severity': 'hard',
                                'code' : error_code,
                                'descr' : "Product %s: %s" % (product_id, error_description)})
        except:
            service = getText(Package.getElementsByTagName('Service')[0].childNodes)
            package_shipment_price = getText(Package.getElementsByTagName('Postage')[0].childNodes)
            #print product_id, service, package_shipment_price
            total = total + float(package_shipment_price)
    
    if len(errors_list) > 0:
        return (0, errors_list)
    else:
        return (1, total)

if __name__ == '__main__':
    print 'Testing USPS handle error'
    print handleUSPSResponse(error)
    print handleUSPSResponse(error_multiple_products)
    
